#
# This file is part of the GROMACS molecular simulation package.
#
# Copyright 2019- The GROMACS Authors
# and the project initiators Erik Lindahl, Berk Hess and David van der Spoel.
# Consult the AUTHORS/COPYING files and https://www.gromacs.org for details.
#
# GROMACS is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2.1
# of the License, or (at your option) any later version.
#
# GROMACS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with GROMACS; if not, see
# https://www.gnu.org/licenses, or write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
#
# If you want to redistribute modifications to GROMACS, please
# consider that scientific software is very special. Version
# control is crucial - bugs must be traceable. We will be happy to
# consider code for inclusion in the official distribution, but
# derived work must not be called official GROMACS. Details are found
# in the README & COPYING files - if they are missing, get the
# official version at https://www.gromacs.org.
#
# To help us fund GROMACS development, we humbly ask that you cite
# the research papers on the package. Check out https://www.gromacs.org.

"""Test exception propagation and inheritance (catchability)"""

# We do not import any gmxapi modules at file scope because one thing we are
# checking for is hidden dependencies.
import pytest

@pytest.mark.usefixtures('cleandir')
def test_catchability():
    """Test exception inheritance of C++ extension module.

    All exceptions should be catchable with the package base exception.
    """
    from gmxapi._gmxapi import Exception as ExtensionException
    import gmxapi
    with pytest.raises(gmxapi.exceptions.Error):
        raise ExtensionException()
    from gmxapi._gmxapi import UnknownException as ExtensionException
    with pytest.raises(gmxapi.exceptions.Error):
        raise ExtensionException()
